﻿using System;
using Vintagestory.API.Common;
using Vintagestory.API.Server;
using Vintagestory.API.Common.Entities;

[assembly: ModInfo( "Lantern",
	Description = "lanterns cast out light",
	Website     = "",
	Authors     = new []{ "OmegaHaxors" } )]

namespace Lantern
{
	public class Lantern: ModSystem
	{
		ICoreAPI api = null;
		ICoreServerAPI sapi = null;
		public override void Start(ICoreAPI api)
		{
			this.api = api;
			base.Start(api);
			api.RegisterEntity("EntityLight",typeof(EntityLight));
		}
		public override void StartServerSide(ICoreServerAPI sapi)
		{
			this.sapi = sapi;
			sapi.World.RegisterGameTickListener(LanternTick,100);
		}
		public void LanternTick(float par)
		{
			IPlayer[] players = sapi.World.AllOnlinePlayers;
			foreach (IPlayer player in players)
			{
				IServerPlayer splayer = player as IServerPlayer;
				if(player.Entity != null && splayer.ConnectionState == EnumClientState.Playing)
				{
					if(player.InventoryManager != null && player.InventoryManager.ActiveHotbarSlot != null)
					{
						ItemSlot slot = player.InventoryManager.ActiveHotbarSlot;
						if(slot.Itemstack != null && slot.Itemstack.Collectible != null)
						{
							if(slot.Itemstack.Collectible.Attributes != null && slot.Itemstack.Collectible.Attributes.KeyExists("lightdistance")){
								CollectibleObject lantern = slot.Itemstack.Collectible;
								byte[] lighthsv = new Byte[]
								{
								7,//hue
								3,//saturation
								5//value
								};//these are default values
								lighthsv = lantern.GetLightHsv(api.World.BlockAccessor,player.Entity.Pos.AsBlockPos,slot.Itemstack);
								SpawnLight(player,lighthsv,slot.Itemstack.Collectible.Attributes["lightdistance"].AsInt());
							}
						}
					}
					if(player.Entity != null && player.Entity.LeftHandItemSlot != null)
					{
						ItemSlot slot = player.Entity.LeftHandItemSlot;
						ItemStack stack = slot.Itemstack;
						if(stack != null && stack.Collectible != null)
						{
							if(stack.Collectible.Attributes != null && stack.Collectible.Attributes.KeyExists("lightdistance")){
							CollectibleObject lantern = stack.Collectible;
								byte[] lighthsv = new Byte[]
								{
								7,//hue
								3,//saturation
								5//value
								};//these are default values
								lighthsv = lantern.GetLightHsv(api.World.BlockAccessor,player.Entity.Pos.AsBlockPos,slot.Itemstack);
								SpawnLight(player,lighthsv,stack.Collectible.Attributes["lightdistance"].AsInt());
							}
						}
					}
				}
			}
		}
		public void SpawnLight(IPlayer player,byte[] lighthsv,int distance)
		{
			int copydistance = 9;
			EntityPos spawnlocnear = player.Entity.Pos.AheadCopy(copydistance);
			EntityProperties type = api.World.GetEntityType(new AssetLocation("lantern:light"));
			Entity lightnear = api.ClassRegistry.CreateEntity(type);
			lightnear.ServerPos.SetPos(spawnlocnear);
			lightnear.ServerPos.SetYaw(0f);
			lightnear.Pos.SetFrom(player.Entity.ServerPos);
			lightnear.PositionBeforeFalling.Set(spawnlocnear);
			lightnear.WatchedAttributes.SetBytes("hsv",lighthsv);
			api.World.SpawnEntity(lightnear);
			distance -= 9;
			
			while(distance > 0)
			{
			copydistance += 4;
			EntityPos spawnloc = player.Entity.Pos.AheadCopy(copydistance);
			Entity light = api.ClassRegistry.CreateEntity(type);
			light.ServerPos.SetPos(spawnloc);
			light.ServerPos.SetYaw(0f);
			light.Pos.SetFrom(player.Entity.ServerPos);
			light.PositionBeforeFalling.Set(spawnloc);
			light.WatchedAttributes.SetBytes("hsv",lighthsv);
			api.World.SpawnEntity(light);
			distance -= 4;
			}	
		}
	}
	public class EntityLight : EntityAgent
	{	
		//private byte[] lightHsv = new byte[] { 7, 3, 2 };
        //public override byte[] LightHsv => this.lightHsv;
		public override void Initialize(EntityProperties properties, ICoreAPI api, long InChunkIndex3d)
		{
			this.LightHsv = this.WatchedAttributes.GetBytes("hsv",new Byte[]{7,3,5});
			base.LightHsv = this.WatchedAttributes.GetBytes("hsv",new Byte[]{7,3,5});
			base.Initialize(properties, api, InChunkIndex3d);
		}
	}

	
}
